install.packages("gridExtra")
install.packages("writexl")
install.packages("factorextra")

library(factoextra)
library(dplyr)
library(tidyr)
library(readxl)
library(tidyverse)
library(cluster)
library(NbClust)
library(ggplot2)
library(writexl)
library(gridExtra)


##########################################
####### SETUP DE LA BASE DE DATOS
#########################################

## establecer directorio de trabajo
setwd("G:/Unidades compartidas/CRC- Indice/Cluster 3.0/Replication files")

## cargar la base de datos con la información de los distintos ejes por municpio
base_otras_var = read.csv("Datos/Base_final_clusters.csv")

base_otras_var <- base_otras_var %>%
  rename(codmpio=ID_MUNICIPIO)%>% # cambiar el nombre de la columna de codigo municipio
  mutate(Cap_4g_pc= CAP_TEC_LTE/POB_2023) # crear la capacidad 4G per cápita por municipio. 

## cargar la base de IPM
IPM <- read_excel("Datos/IPM.xlsx", sheet = "IPM - 2018") %>% 
  subset(select = c(IPM, codmpio))# Cargar los datos de IPM por municipio

# juntar ambas bases. 
base_var_ipm <- left_join(x =IPM, y= base_otras_var, by= "codmpio" ) %>% ## juntar ambas bases
  subset(select= c(IPM, codmpio, NUM_PROV, 
                   DENS_POB, DIST_CAPITAL, CEN_POB_RURAL_DIS,
                   Cap_4g_pc, INDICADOR_LTE)) # seleccionar las variables de interés correspondientes a los ejes


## anadir usuarios moviles por municipio, velocidad de descarga y carga, tasa perdida de paquetes 
# 1. usuarios mov por munic (Solo hay 2023 2T)
us_por_munic = read_excel("Datos/Líneas móviles por municipio.xlsx") # cargar la base de lineas móviles

us_por_munic <- us_por_munic %>% 
  filter(ANNO==2023 & TRIMESTRE==2 & MES_DEL_TRIMESTRE==3)%>% # mantener junio del 2023 
  group_by(ID_MUNICIPIO) %>% # agrupar por municipio los datos por proveedor
  summarise(lineas_en_serv_promedio= sum(TOTAL_LINEAS_EN_SERV_TRAFICO))%>% # sumar las lineas por provedor para cada muniucipio
  rename(codmpio=ID_MUNICIPIO)# cambiar el nombre de la variable. 

# unir a la base anterior. 
base_var_ipm <- base_var_ipm %>%left_join(y= us_por_munic, by= "codmpio")

# 2. Velocidades, tasa de pérdida de paquetes, Jitter y Latencia 
velocidades = read_excel("Datos/Prom_pond_linea_base.xlsx")
velocidades$Jitter <- as.numeric(velocidades$Jitter)

velocidades <- velocidades %>% rename(codmpio=ID_MUNICIPIO) # cambiar el nombre de la variable cod municipio

# unir a la base consolidada.
base_var_ipm <- base_var_ipm %>%left_join(y= velocidades, by= "codmpio" )

#exportar la base lista si se quiere. 
#write.csv(base_var_ipm, "Output/base_cluster_CRC3_Extra.csv")

# revisión de valores missing
colSums(is.na(base_var_ipm))
prop.table(colSums(is.na(base_var_ipm)))


## revisar casos completos de observaciones. 
base_var_ipm_completa <- base_var_ipm[complete.cases(base_var_ipm),]


############################################
##### Realizar el análisis de cluster
############################################


#################################
### Municipios excluidos
################################

excluidos = read_excel("Datos/320 municipios exceptuados - Clúster CRC.xlsx") %>% # leer excel con municipios excluidos 
  rename(codmpio=DIVIPOLA)%>% # cambiar el nombre de la variable municipio
  subset(select=c(codmpio,Exceptuados)) # seleccionar las variables para el pegue. 

base_var_ipm_completa <- left_join(base_var_ipm_completa, excluidos, by="codmpio")%>% # juntar las bases 
  mutate(Exceptuados= ifelse(is.na(Exceptuados), 0,Exceptuados))# añadir 0 para aquellos que no son exceptuados

## limitar a los excluidos
base_var_ipm_completa <- base_var_ipm_completa %>% 
  filter(Exceptuados==0)%>% subset(select=-Exceptuados) #eliminar la variable exceptuados para que no se use en ANS


#########################################
# Clusterización
#########################################


# Seleccion de variables a trabaja
X <- base_var_ipm_completa %>% 
  select(-c("codmpio", "TestCount_UL", "TestCount_PG",
                                         "TestCount_JT", "TestCount_PL","TestCount_DL"))# eliminar variables de pruebas

Y <- base_var_ipm_completa %>% select(c("codmpio")) # seleccionar los códigos del municipio

X <- X %>%
  mutate_all(as.numeric) # convertir todas las variables en numéricas. 
### se escalan las variables.
X_scale<-scale(X)

#########################################
# Análisis del número óptimo de categorías
#########################################
# método de explicación de la suma del error al interior de los clúster. 

grafica2 <- fviz_nbclust(X_scale, pam, method = "wss", k.max = 10)


# Cambiar el título y el tamaño de letra
grafica2 <- grafica2 +
  ggtitle("Método del Codo") + xlab("Numero de Clusters K") +  # Cambia "Título Eje X" por el título que desees para el eje X
  ylab("Suma Cuadrada Total al Interior") + 
  theme(plot.title = element_text(size = 20),
        axis.title.x = element_text(size = 20), # Ajusta el tamaño y la negrita del eje X
        axis.title.y = element_text(size = 20),  # Ajusta el tamaño y la negrita del eje Y
        axis.text.x = element_text(size = 16),  # Ajusta el tamaño de los valores del eje X
        axis.text.y = element_text(size = 16))  # Ajusta
print(grafica2)


### aplicación de la metodología de Gap Statistic Hastie (2001)
#calcular la estadística de brecha entre la varianza explicada de los datos 
# y de una función uniforme para definir el número de clústeres
gap_stat <- clusGap (X_scale,
                     FUNcluster = pam,
                     K.max = 10, # clústeres máximos a considerar 
                     B = 10) # iteraciones totales de arranque

# trazar el número de conglomerados frente a la estadística computada
grafica <- fviz_gap_stat(gap_stat)
# Cambiar el título y el tamaño de letra
grafica <- grafica +
  ggtitle("Gap Statistic") + xlab("Numero de Clusters K") +  # Cambia "Título Eje X" por el título que desees para el eje X
  ylab("Gap statistic (k)") + 
  theme(plot.title = element_text(size = 20),
        axis.title.x = element_text(size = 20), # Ajusta el tamaño y la negrita del eje X
        axis.title.y = element_text(size = 20),  # Ajusta el tamaño y la negrita del eje Y
        axis.text.x = element_text(size = 16),  # Ajusta el tamaño de los valores del eje X
        axis.text.y = element_text(size = 16))  # Ajusta
print(grafica)


# combinar ambas gráficas en una misma. 
graficas_combinadas <- grid.arrange(grafica2, grafica, ncol = 2)



#########################################
# Algoritmo de segmentación
#########################################
## aplicar los 4 clusters en k-medioides
set.seed(42) # semilla para la replicabilidad. 
kmed <- pam(X_scale,k=7)



#######################################################################################
#### Extraer la tabla de estadísticas descriptivas para la interpretación de los cluster
#######################################################################################

### añadir el resultado de la segmentación a una nueva tabla que se llama estadisticas
# Paso 1: Añadir el resultado de la segmentación a una nueva tabla llamada estadisticas
estadisticas <- base_var_ipm_completa %>% 
  mutate(kmeans_nuevo = kmed$clustering)

# Paso 2: Agrupar por la variable de segmentación y calcular estadísticas ponderadas
estadisticas <- estadisticas %>%
  group_by(kmeans_nuevo) %>%
  summarise(
    # Contar el número de observaciones en cada segmento
    num_obs = n(),
    # Calcular la media ponderada para la velocidad de descarga
    Descarga = weighted.mean(Descarga, TestCount_DL),
    # Calcular la media ponderada para la velocidad de carga
    Carga = weighted.mean(Carga, TestCount_UL),
    # Calcular la media ponderada para la variabilidad de la conexión (Jitter)
    Jitter = weighted.mean(as.numeric(Jitter), as.numeric(TestCount_JT)),
    # Calcular la media ponderada para la latencia
    Latencia = weighted.mean(Latencia, TestCount_PG),
    # Calcular la media ponderada para la pérdida de paquetes
    Perdida_paquetes = weighted.mean(Perdida_de_Paquetes, TestCount_PL),
    # Calcular la proporción media de penetración 4G
    Ind_penetracion_4G = mean(INDICADOR_LTE),
    # Calcular la capacidad promedio de la red 4G
    CAPACIDAD_4G_HAB = mean(Cap_4g_pc),
    # Calcular la media de la población rural
    Pob_rural = mean(CEN_POB_RURAL_DIS),
    # Calcular la densidad de población media
    Dens_POB = mean(DENS_POB),
    # Calcular la media del Índice de Pobreza Multidimensional (IPM)
    IPM = mean(IPM),
    # Calcular la distancia media a la capital
    distancia_capital = mean(DIST_CAPITAL),
    # Calcular la cantidad media de proveedores
    Num_provedores = mean(NUM_PROV),
    # Calcular la cantidad media de líneas de servicio
    Lineas_servicio = mean(lineas_en_serv_promedio)
  ) %>%
  # Paso 3: Reorganizar los datos para tener la tabla en formato wide con columnas por variables. 
  gather(key = "variable", "value", num_obs, Descarga, Carga, Perdida_paquetes, Jitter, Latencia, Ind_penetracion_4G, CAPACIDAD_4G_HAB,
         Pob_rural, IPM, distancia_capital, Num_provedores, Lineas_servicio, Dens_POB) %>%
  spread(kmeans_nuevo, value) %>%  group_by(variable)

###### extraer estadísticas #######
# Redondear todas las columnas numéricas a 2 decimales
estadisticas <- estadisticas %>% mutate(across(where(is.numeric), ~round(., 2)))


write_xlsx(estadisticas, "Output/estadisticas_exceptuados_7_cluster_5_var.xlsx")


###### extraer la lista de municipios #######
### columna de resultados
resultados_cluster <- base_var_ipm_completa %>% 
  mutate(Cluster = kmed$clustering) %>% # añadir resultado de segmentación
  subset(select=c(codmpio, Cluster)) # seleccionar las variables que se necesitan para la lista. 


# juntar todos los municipios 
lista_regulacion <- left_join(base_otras_var, resultados_cluster, by="codmpio")%>% # juntar otros municipios que tenían NA o son exceptuados de las obligaciones de calidad
  subset(select=c(DPNOM, MPIO, codmpio, Cluster)) # seleccionar las 4 variables necesarias  
# reemplazar los municipios que están excluidos

# juntar con la lista de excluidos y añadir la variable de si el municipio está excluido. 
lista_regulacion_excluidos <- left_join(lista_regulacion, excluidos, by="codmpio")%>% 
  mutate(Exceptuados= ifelse(is.na(Exceptuados), 0,Exceptuados))

## exportar la lista:
writexl::write_xlsx(lista_regulacion_excluidos, "Output/lista_5var_7cluster_exceptuados.xlsx")



